__title__ = 'tqt'
__description__ = 'torch implement of TQT'
__version__ = '0.4.2'
__author__ = 'Pannenets F'
__author_email__ = 'pannenets.f@foxmail.com'
__license__ = 'BSD-3-Clause License'
__url__ = 'https://github.com/PannenetsF/TQT'
